/**
 * 
 */
package lcsb.mapviewer.converter.model.sbgnml;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.charset.StandardCharsets;

import javax.xml.bind.JAXBException;

import org.apache.commons.io.IOUtils;
import org.apache.commons.text.StringEscapeUtils;
import org.apache.log4j.Logger;
import org.apache.log4j.spi.LoggingEvent;
import org.sbgn.SbgnUtil;
import org.sbgn.bindings.Sbgn;

import lcsb.mapviewer.common.EventStorageLoggerAppender;
import lcsb.mapviewer.common.MimeType;
import lcsb.mapviewer.converter.ConverterException;
import lcsb.mapviewer.converter.ConverterParams;
import lcsb.mapviewer.converter.IConverter;
import lcsb.mapviewer.converter.InvalidInputDataExecption;
import lcsb.mapviewer.model.map.InconsistentModelException;
import lcsb.mapviewer.model.map.model.Model;

/**
 * @author Michał Kuźma
 *
 */
public class SbgnmlXmlConverter implements IConverter {
  Logger logger = Logger.getLogger(SbgnmlXmlConverter.class);

  @Override
  public Model createModel(ConverterParams params) throws InvalidInputDataExecption {
    SbgnmlXmlParser parser = new SbgnmlXmlParser();
    return parser.createModel(params);
  }

  @Override
  public InputStream exportModelToInputStream(Model model) throws InconsistentModelException, ConverterException {
    EventStorageLoggerAppender appender = new EventStorageLoggerAppender();
    Sbgn exportedData;
    StringBuilder notes = new StringBuilder("");
    try {
      Logger.getRootLogger().addAppender(appender);
      SbgnmlXmlExporter exporter = new SbgnmlXmlExporter();
      exportedData = exporter.toSbgnml(model);
      if (!appender.getWarnings().isEmpty()) {
        for (LoggingEvent event : appender.getWarnings()) {
          if (event.getMessage() instanceof String) {
            notes.append("\n" + ((String) event.getMessage()));
          }
        }
      }
    } finally {
      Logger.getRootLogger().removeAppender(appender);
    }
    try {
      File tempFile = File.createTempFile("sbgnmlExportTempFile", ".tmp");
      InputStream result;
      SbgnUtil.writeToFile(exportedData, tempFile);
      try (FileInputStream inputStream = new FileInputStream(tempFile)) {
        String xml = IOUtils.toString(inputStream, "UTF-8");
        if (notes.length() > 0) {
          int position = xml.indexOf("<map language");
          if (position < 0) {
            throw new ConverterException("problematic output sbgn. Cannot find map tag");
          }
          String notesNode = "<notes><html:body xmlns:html=\"http://www.w3.org/1999/xhtml\">" +
              StringEscapeUtils.escapeXml10(notes.toString()) + "\n</html:body></notes> ";
          xml = xml.substring(0, position) + notesNode + xml.substring(position, xml.length());
        }
        result = new ByteArrayInputStream(xml.getBytes(StandardCharsets.UTF_8));
      }
      return result;
    } catch (IOException e) {
      throw new ConverterException(e);
    } catch (JAXBException e) {
      throw new ConverterException(e);
    }
  }

  @Override
  public File exportModelToFile(Model model, String filePath) throws InconsistentModelException, ConverterException {
    InputStream inputStream = exportModelToInputStream(model);

    try {
      File exportedFile = new File(filePath);
      if (exportedFile.getParentFile() != null) {
        exportedFile.getParentFile().mkdirs();
      }
      OutputStream outputStream = new FileOutputStream(exportedFile);
      IOUtils.copy(inputStream, outputStream);
      outputStream.close();

      return exportedFile;
    } catch (IOException e) {
      throw new ConverterException(e);
    }
  }

  @Override
  public String getCommonName() {
    return "SBGN-ML";
  }

  @Override
  public MimeType getMimeType() {
    return MimeType.TEXT;
  }

  @Override
  public String getFileExtension() {
    return "sbgn";
  }

}
