package lcsb.mapviewer.converter.graphics.bioEntity.element.species;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import lcsb.mapviewer.commands.ColorExtractor;
import lcsb.mapviewer.converter.graphics.ConverterParams;
import lcsb.mapviewer.model.map.species.Complex;

public class ComplexConverterTest {
  Logger logger = Logger.getLogger(ComplexConverterTest.class);

  ColorExtractor colorExtractor = new ColorExtractor(Color.RED, Color.GREEN, Color.BLUE);

  @Before
  public void setUp() throws Exception {
  }

  @After
  public void tearDown() throws Exception {
  }

  @Test
  public void testDrawText() throws Exception {
    try {
      int size = 200;
      BufferedImage bi = new BufferedImage(size, size, BufferedImage.TYPE_INT_ARGB);
      Graphics2D graphics = bi.createGraphics();
      ComplexConverter rc = new ComplexConverter(colorExtractor);

      Complex alias = createComplex();
      rc.drawText(alias, graphics, new ConverterParams());

      boolean onlyWhite = true;
      for (int x = 0; x < size; x++) {
        for (int y = 0; y < size; y++) {
          if (bi.getRGB(x, y) != 0) {
            onlyWhite = false;
          }
        }
      }
      assertFalse(onlyWhite);

    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }

  @Test
  public void testDrawTooBigText() throws Exception {
    try {
      int size = 200;
      double scale = 100;
      BufferedImage bi = new BufferedImage(size, size, BufferedImage.TYPE_INT_ARGB);
      Graphics2D graphics = bi.createGraphics();
      Color color = graphics.getColor();
      graphics.scale(1.0 / scale, 1.0 / scale);
      ComplexConverter rc = new ComplexConverter(colorExtractor);

      Complex alias = createComplex();
      rc.drawText(alias, graphics, new ConverterParams());

      assertEquals(color, graphics.getColor());

    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }

  private Complex createComplex() {
    Complex protein = new Complex("id");
    protein.setName("NAME_OF_THE_ELEMENT");
    protein.setX(10);
    protein.setY(20);
    protein.setWidth(100);
    protein.setHeight(80);
    protein.setColor(Color.WHITE);

    return protein;
  }
}
