package lcsb.mapviewer.converter.model.sbml;

import static org.junit.Assert.assertEquals;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;

import org.apache.log4j.Logger;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;

import lcsb.mapviewer.converter.ConverterParams;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.model.ModelComparator;
import lcsb.mapviewer.model.map.model.ModelFullIndexed;
import lcsb.mapviewer.model.map.species.Element;
import lcsb.mapviewer.model.map.species.GenericProtein;
import lcsb.mapviewer.model.map.species.Species;

@RunWith(Parameterized.class)
public class ElementPropertiesExportToMultiTest {

  static Logger logger = Logger.getLogger(ElementPropertiesExportToMultiTest.class.getName());

  private Model model;

  public ElementPropertiesExportToMultiTest(String propertyName, Model model) {
    this.model = model;
  }

  @Parameters(name = "{index} : {0}")
  public static Collection<Object[]> data() throws IOException {
    Collection<Object[]> data = new ArrayList<>();

    data.add(createTestEntry("Empty", createElement()));

    Species element = createElement();
    element.setAbbreviation("xyz");
    data.add(createTestEntry("Abbreviation", element));

    element = createElement();
    element.setFormerSymbols(Arrays.asList(new String[] { "symbol 1", "s2" }));
    data.add(createTestEntry("Former Symbols", element));

    element = createElement();
    element.setFormula("C2H5OH");
    data.add(createTestEntry("Formula", element));

    element = createElement();
    element.setFullName("Amazing element");
    data.add(createTestEntry("Full name", element));

    element = createElement();
    element.setHomodimer(4);
    data.add(createTestEntry("Homodimer", element));

    element = createElement();
    element.setCharge(5);
    data.add(createTestEntry("Charge", element));

    element = createElement();
    element.setHypothetical(true);
    data.add(createTestEntry("Hypothetical", element));

    element = createElement();
    element.setActivity(true);
    data.add(createTestEntry("Activity", element));

    element = createElement();
    element.setSymbol("H2O");
    data.add(createTestEntry("Symbol", element));

    element = createElement();
    element.setSynonyms(Arrays.asList(new String[] { "syn 1", "s2" }));
    data.add(createTestEntry("Synonyms", element));

    return data;
  }

  private static Object[] createTestEntry(String string, Element element) {
    Model result = new ModelFullIndexed(null);
    result.setIdModel("X");
    result.setWidth(200);
    result.setHeight(200);
    result.addElement(element);
    return new Object[] { string, result };
  }

  private static Species createElement() {
    Species element = new GenericProtein("id");
    element.setX(10);
    element.setY(10);
    element.setWidth(1000);
    element.setHeight(30);
    return element;
  }

  @Test
  public void createModelTest() throws Exception {
    try {
      SbmlExporter sbmlExporter = new SbmlExporter();
      sbmlExporter.setProvideDefaults(false);
      SbmlParser sbmlParser = new SbmlParser();
      String xmlString = sbmlExporter.toXml(model);

      InputStream is = new ByteArrayInputStream(xmlString.getBytes("UTF-8"));

      Model model2 = sbmlParser.createModel(new ConverterParams().inputStream(is).sizeAutoAdjust(false));
      model2.setName(model.getName());

      ModelComparator comparator = new ModelComparator();
      assertEquals(0, comparator.compare(model, model2));

    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }

}
