package lcsb.mapviewer.converter.model.sbml;

import static org.junit.Assert.assertEquals;

import java.io.ByteArrayInputStream;

import org.apache.log4j.Logger;
import org.junit.Test;

import lcsb.mapviewer.converter.ConverterParams;
import lcsb.mapviewer.converter.model.celldesigner.CellDesignerXmlParser;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.model.ModelComparator;
import lcsb.mapviewer.model.map.reaction.Reaction;

public class SbmlExporterFromCellDesignerTest extends SbmlTestFunctions {
  Logger logger = Logger.getLogger(SbmlExporterFromCellDesignerTest.class);
  SbmlParser parser = new SbmlParser();
  SbmlExporter exporter = new SbmlExporter();

  CellDesignerXmlParser cellDesignerXmlParser = new CellDesignerXmlParser();

  ModelComparator comparator = new ModelComparator();

  @Test
  public void testExportHeterodimerAssociation() throws Exception {
    try {
      Model originalModel = cellDesignerXmlParser
          .createModel(new ConverterParams().filename("testFiles/cell_designer_problems/heterodimer_association.xml"));
      Model model = getModelAfterSerializing(originalModel);
      Reaction reaction1 = originalModel.getReactions().iterator().next();
      Reaction reaction2 = model.getReactions().iterator().next();
      // change reaction id - due to some issues it cannot be persisted properly in
      // sbml format
      reaction2.setIdReaction(reaction1.getIdReaction());

      model.setName(originalModel.getName());

      String cellDesignerXml = cellDesignerXmlParser.toXml(model);
      ByteArrayInputStream stream = new ByteArrayInputStream(cellDesignerXml.getBytes("UTF-8"));

      assertEquals(0, comparator.compare(model, originalModel));
      Model cdModel = cellDesignerXmlParser.createModel(new ConverterParams().inputStream(stream));

      cdModel.setName(originalModel.getName());

      assertEquals(0, comparator.compare(model, cdModel));
    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }

  private Model getModelAfterSerializing(Model originalModel) throws Exception {
    String xml = exporter.toXml(originalModel);
    // logger.debug(xml);
    ByteArrayInputStream stream = new ByteArrayInputStream(xml.getBytes("UTF-8"));
    Model result = parser.createModel(new ConverterParams().inputStream(stream));
    // showImage(originalModel);
    // showImage(result);
    return result;
  }

}
