package lcsb.mapviewer.persist;

import com.mchange.v2.c3p0.ComboPooledDataSource;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Environment;
import org.springframework.boot.orm.jpa.hibernate.SpringPhysicalNamingStrategy;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.jdbc.datasource.init.DataSourceInitializer;
import org.springframework.jdbc.datasource.init.DatabasePopulator;
import org.springframework.orm.hibernate5.HibernateTransactionManager;
import org.springframework.orm.hibernate5.LocalSessionFactoryBean;
import org.springframework.security.authentication.encoding.Md5PasswordEncoder;
import org.springframework.transaction.annotation.EnableTransactionManagement;

import javax.sql.DataSource;
import java.beans.PropertyVetoException;
import java.util.Properties;

@Configuration
@EnableTransactionManagement
@ComponentScan(basePackages = {"lcsb.mapviewer.persist"})
public class SpringPersistConfig {

    @Bean
    public ConfigurationHolder config() {
        return new ConfigurationHolder();
    }

//  @Bean
//  public HikariDataSource dataSource(ConfigurationHolder config) {
//    HikariDataSource dataSource = new HikariDataSource();
//    dataSource.setDriverClassName("org.postgresql.Driver");
//    dataSource.setJdbcUrl(config.getDbUri());
//    dataSource.setUsername(config.getDbUsername());
//    dataSource.setPassword(config.getDbPassword());
//    dataSource.setIdleTimeout(30000);
//    dataSource.setMaximumPoolSize(90);
//    dataSource.setConnectionTestQuery("select 1");
//    return dataSource;
//  }

    @Bean(destroyMethod = "close")
    public ComboPooledDataSource dataSource(ConfigurationHolder config) throws PropertyVetoException {
        ComboPooledDataSource dataSource = new ComboPooledDataSource();
        dataSource.setDriverClass("org.postgresql.Driver");
        dataSource.setJdbcUrl(config.getDbUri());
        dataSource.setUser(config.getDbUsername());
        dataSource.setPassword(config.getDbPassword());
        dataSource.setMinPoolSize(5);
        dataSource.setMaxPoolSize(90);
        dataSource.setMaxStatements(5);
        return dataSource;
    }

    @Bean
    public LocalSessionFactoryBean sessionFactory(DataSource dataSource) {
        Properties props = new Properties();
        props.setProperty(Environment.DIALECT, "org.hibernate.dialect.PostgreSQL95Dialect");
        props.setProperty(Environment.SHOW_SQL, "false");
        LocalSessionFactoryBean factoryBean = new LocalSessionFactoryBean();
        factoryBean.setDataSource(dataSource);
        factoryBean.setHibernateProperties(props);
        factoryBean.setPackagesToScan("lcsb.mapviewer.model");
        factoryBean.setImplicitNamingStrategy(new CustomImplicitNamingStrategy());
        factoryBean.setPhysicalNamingStrategy(new SpringPhysicalNamingStrategy());
        return factoryBean;
    }

    @Bean
    public HibernateTransactionManager transactionManager(SessionFactory sessionFactory) {
        HibernateTransactionManager transactionManager = new HibernateTransactionManager();
        transactionManager.setSessionFactory(sessionFactory);
        return transactionManager;
    }

    @Bean
    public Md5PasswordEncoder passwordEncoder() {
        return new Md5PasswordEncoder();
    }

    @Bean
    public DataSourceInitializer dataSourceInitializer(DataSource dataSource, DatabasePopulator databasePopulator) {
        DataSourceInitializer initializer = new DataSourceInitializer();
        initializer.setDataSource(dataSource);
        initializer.setDatabasePopulator(databasePopulator);
        initializer.setEnabled(true);
        return initializer;
    }

}
