package lcsb.mapviewer.api.convert;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Map;

import javax.xml.stream.XMLStreamException;

import org.apache.log4j.Logger;
import org.sbml.jsbml.SBMLException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CookieValue;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import lcsb.mapviewer.api.BaseController;
import lcsb.mapviewer.api.ObjectNotFoundException;
import lcsb.mapviewer.api.QueryException;
import lcsb.mapviewer.common.Configuration;
import lcsb.mapviewer.converter.ConverterException;
import lcsb.mapviewer.converter.InvalidInputDataExecption;
import lcsb.mapviewer.converter.graphics.DrawingException;
import lcsb.mapviewer.model.map.InconsistentModelException;
import lcsb.mapviewer.services.SecurityException;

@RestController
public class ConvertController extends BaseController {
  @SuppressWarnings("unused")
  private Logger logger = Logger.getLogger(ConvertController.class);

  @Autowired
  private ConvertRestImpl convertController;
  
 
  @RequestMapping(value = "/convert/{fromFormat}:{toFormat}", 
		  produces = {MediaType.APPLICATION_XML_VALUE}, 
		  method = { RequestMethod.POST })
  public String convertInput(
	  @PathVariable(value = "fromFormat") String fromFormat,
	  @PathVariable(value = "toFormat") String toFormat,
      @RequestBody String body,
      @CookieValue(value = Configuration.AUTH_TOKEN) String token
  ) 
		  throws SecurityException, IOException, QueryException, SBMLException,
		  InvalidInputDataExecption, InconsistentModelException, ConverterException, 
		  XMLStreamException, DrawingException 
  {    
    return convertController.convert(token, fromFormat, toFormat, body);

  }
  
  @RequestMapping(value = "/convert/image/{fromFormat}:{toFormat}", method = { RequestMethod.POST })
  public @ResponseBody ResponseEntity<byte[]> convertInputToImage(
	  @PathVariable(value = "fromFormat") String fromFormat,
	  @PathVariable(value = "toFormat") String toFormat,
	  //@RequestParam(value = "targetWidth", defaultValue = "0") Double targetWidth,
	  //@RequestParam(value = "targetHeight", defaultValue = "0") Double targetHeight,
      @RequestBody String body,
      @CookieValue(value = Configuration.AUTH_TOKEN) String token
  ) 
		  throws SecurityException, IOException, QueryException, SBMLException,
		  InvalidInputDataExecption, InconsistentModelException, ConverterException, 
		  XMLStreamException, DrawingException 
  {
	  ByteArrayOutputStream os = convertController.converToImage(token, fromFormat, toFormat, body);
	  
	  return ResponseEntity.ok().contentLength(os.size())
    		.contentType(MediaType.APPLICATION_OCTET_STREAM)
    		.header("Content-Disposition", "attachment; filename=model" + toFormat )
    		.body(os.toByteArray());
  }
  
  @RequestMapping(value = "/convert/", method = { RequestMethod.GET, RequestMethod.POST }, produces = {
	      MediaType.APPLICATION_JSON_VALUE })
  public Map<String, Object> getInformation(
		  @CookieValue(value = Configuration.AUTH_TOKEN) String token
		  ) throws SecurityException, ObjectNotFoundException
  {
	  return convertController.getInformation(token);
  }
  
  @RequestMapping(value = "/convert/image/", method = { RequestMethod.GET, RequestMethod.POST }, produces = {
	      MediaType.APPLICATION_JSON_VALUE })
  public Map<String, Object> getInformationImage(
		  @CookieValue(value = Configuration.AUTH_TOKEN) String token
		  ) throws SecurityException, ObjectNotFoundException
  {	  
	  return convertController.getInformationImage(token);
  }
  
}