package lcsb.mapviewer.api.convert;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.BufferedWriter;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.xml.stream.XMLStreamException;

import org.apache.commons.io.FileUtils;
import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.Test;
import org.sbml.jsbml.SBMLException;
import org.springframework.beans.factory.annotation.Autowired;


import lcsb.mapviewer.api.QueryException;
import lcsb.mapviewer.api.RestTestFunctions;
import lcsb.mapviewer.converter.ConverterException;
import lcsb.mapviewer.converter.InvalidInputDataExecption;
import lcsb.mapviewer.converter.graphics.DrawingException;
import lcsb.mapviewer.model.map.InconsistentModelException;
import lcsb.mapviewer.services.SecurityException;

public class ConvertRestImplTest extends RestTestFunctions {

  @SuppressWarnings("unused")
  private Logger logger = Logger.getLogger(ConvertRestImplTest.class);

  @Autowired
  public ConvertRestImpl convertRestImpl;

  @AfterClass
  public static void tearDownAfterClass() throws Exception {
  }

  @Before
  public void setUp() throws Exception {
  }

  @After
  public void tearDown() throws Exception {
  }
  
  @SuppressWarnings("unchecked")  
  @Test
  public void testGetInformation() throws Exception {
    try {
    	Map<String, Object> result = convertRestImpl.getInformationImage(token);
    	assertTrue(result.keySet().size() == 2);
    	assertNotNull(result.get("inputs"));
    	assertNotNull(result.get("outputs"));
    	
    	List<Object> inputs = (List<Object>) result.get("inputs");
    	List<Object> outputs = (List<Object>) result.get("outputs");
    	
    	assertTrue(inputs.size() > 0);    	
    	assertTrue(outputs.size() > 0);    	
    	 
    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }
  
  @SuppressWarnings("unchecked")
  @Test
  public void testGetInformationImage() throws Exception {
    try {
    	Map<String, Object> result = convertRestImpl.getInformationImage(token);
    	assertTrue(result.keySet().size() == 2);
    	assertNotNull(result.get("inputs"));
    	assertNotNull(result.get("outputs"));
    	
    	List<Object> inputs = (List<Object>) result.get("inputs");
    	List<Object> outputs = (List<Object>) result.get("outputs");
    	
    	assertTrue(inputs.size() > 0);    	
    	assertTrue(outputs.size() > 0);
    	 
    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }


  @Test
  public void testCelDesigner2Sbml1() throws Exception {
    try {
    	File file = new File("testFiles/convert/sample-cd.xml");
    	String content = FileUtils.readFileToString(file);
    	String result = convertRestImpl.convert(token, "lcsb.mapviewer.converter.model.celldesigner.CellDesignerXmlParser", 
    			"lcsb.mapviewer.converter.model.sbml.SbmlParser", content);
    	assertTrue(result.length() > 0);
    	assertTrue(result.contains("</layout:speciesReferenceGlyph>"));
    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }
  
  /*
   * this test does not work because the generator is not deterministic
  @Test
  public void testCelDesigner2Sbml2() throws Exception {
    try {
    	File file = new File("testFiles/convert/sample-cd.xml");
    	String content = FileUtils.readFileToString(file);
    	String result = convertRestImpl.convert(token, "CellDesigner_SBML", "SBML", content);
    	//String result2 = convertRestImpl.convert(token, "CellDesigner_SBML", "SBML", content);
    	String result2 = convertRestImpl.convert(token, "lcsb.mapviewer.converter.model.celldesigner.CellDesignerXmlParser", 
    			"lcsb.mapviewer.converter.model.sbml.SbmlParser", content);    	
    	
    	assertTrue(result.equals(result2));    	    	
    	
    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }
  */
 
  @Test
  public void testCelDesigner2Svg() throws Exception {
    try {
    	File file = new File("testFiles/convert/sample-cd.xml");
    	String content = FileUtils.readFileToString(file);
    	String result = convertRestImpl.converToImage(token, "CellDesigner_SBML", "svg", content).toString();
    	
    	 // FileOutputStream outputStream = new FileOutputStream("sample-cd.png");
    	// outputStream.write(convertRestImpl.converToImage(token, "CellDesigner_SBML", "png", content).toByteArray());
    	// outputStream.close();
        
    	assertTrue(result.contains("<rect"));    	
    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }
  
  @Test
  public void testCellDesigner2Sbml2Svg() throws Exception {
    try {
        File file = new File("testFiles/convert/neuron_cd.xml");
        String content = FileUtils.readFileToString(file);
        String resultSbml = convertRestImpl.convert(token, "CellDesigner_SBML", "SBML", content).toString();
        String resultSvg = convertRestImpl.converToImage(token, "SBML", "svg", resultSbml).toString();
        
         // FileOutputStream outputStream = new FileOutputStream("sample-cd.png");
        // outputStream.write(convertRestImpl.converToImage(token, "CellDesigner_SBML", "png", content).toByteArray());
        // outputStream.close();
        
        assertTrue(resultSvg.contains("<rect"));        
    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }
  
  @Test
  public void testCellDesigner2Sbml2CellDesignerSvg() throws Exception {
    try {
        File file = new File("testFiles/convert/neuron_cd.xml");
        String content = FileUtils.readFileToString(file);
        String resultSbml = convertRestImpl.convert(token, "CellDesigner_SBML", "SBML", content).toString();
        String resultCellDesignerSbml = convertRestImpl.convert(token, "SBML", "CellDesigner_SBML", resultSbml).toString();
        String resultSvg = convertRestImpl.converToImage(token, "CellDesigner_SBML", "svg", resultCellDesignerSbml).toString();
        
         // FileOutputStream outputStream = new FileOutputStream("sample-cd.png");
        // outputStream.write(convertRestImpl.converToImage(token, "CellDesigner_SBML", "png", content).toByteArray());
        // outputStream.close();
        
        assertTrue(resultSvg.contains("<rect"));        
    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }
  
  @Test
  public void testScaling() throws Exception {
    try {
    	File file = new File("testFiles/convert/sample-cd.xml");
    	String content = FileUtils.readFileToString(file);
    	String result1 = convertRestImpl.converToImage(token, "CellDesigner_SBML", "svg", content, 50.0, 50.0).toString();
    	String result2 = convertRestImpl.converToImage(token, "CellDesigner_SBML", "svg", content, 100.0, 100.0).toString();
       
    
    	Pattern pattern = Pattern.compile(".*scale\\(([0-9]+\\.[0-9]+),([0-9]+\\.[0-9]+)\\).*");
    	Matcher matcher1 = pattern.matcher(result1);
    	Matcher matcher2 = pattern.matcher(result2);
    	
    	assertTrue(matcher1.find());    	
    	assertTrue(matcher2.find());
    	
    	assertFalse(matcher1.group(1).toString().equals(matcher2.group(1).toString()));    	
    	
    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }
  
  @Test
  public void test1() throws Exception {
    try {
    	File file = new File("testFiles/convert/glycolysis2.xml");
        String content = FileUtils.readFileToString(file);
        
        String result = convertRestImpl.convert(token, "CellDesigner_SBML", "SBML", content);
        ByteArrayOutputStream result1 = convertRestImpl.converToImage(token, "SBML", "svg", result);
        
        File output = new File("testFiles/convert/glycolysis2.svg");
            
        FileOutputStream outputStream = new FileOutputStream(output);
        outputStream.write(result1.toByteArray());
        outputStream.close();
        assertTrue(output.exists());
        output.delete();
    	
            	
    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }
  
  @Test
  public void test2() throws Exception {
    try {
    	File file = new File("testFiles/convert/glycolysis2.xml");
    	String content = FileUtils.readFileToString(file);
    	
    	String result = convertRestImpl.convert(token, "CellDesigner_SBML", "SBML", content);
    	String result1 = convertRestImpl.convert(token, "SBML", "CellDesigner_SBML", result);
    	
            	
    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }
  
  
  @Test
  public void testCelDesigner2Sbgn() throws Exception {
    try {
    	File file = new File("testFiles/convert/sample-cd.xml");
    	String content = FileUtils.readFileToString(file);
    	String result = convertRestImpl.convert(token, "CellDesigner_SBML", "SBGN-ML", content).toString();    	    	
    	assertTrue(result.contains("glyph class=\"complex\""));    	
    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }
  
  /*
   * this test does not work because the generator is not deterministic
  @Test
  public void testCelDesigner2Svg2() throws Exception {
    try {
    	File file = new File("testFiles/convert/sample-cd.xml");
    	String content = FileUtils.readFileToString(file);
    	String result = convertRestImpl.converToImage(token, "CellDesigner_SBML", "svg", content).toString();    	
    	String result2 = convertRestImpl.converToImage(token, "CellDesigner_SBML", 
    			"lcsb.mapviewer.converter.graphics.SvgImageGenerator", content).toString();
    	
    	assertTrue(result.equals(result2));
    	
    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    }
  }
  */
  
  @SuppressWarnings("unchecked")
  private List<String> extractIdsFromOutputs(Map<String, Object> infoObject) {
	  List<String> ids = new ArrayList<>();
	  for (Map<String, List<String>> o : (List<Map<String, List<String>>>) infoObject.get("outputs")) {
		  ids.add(o.get("available_names").get(0));
	  }
	  return ids;	  
  }
      
  private void test2All(String content, String converter) 
		throws SBMLException, SecurityException, InvalidInputDataExecption, InconsistentModelException, 
		IOException, ConverterException, XMLStreamException, DrawingException, QueryException 
  {
	  
	  Map<String, Object> info = convertRestImpl.getInformation(token);
	  Map<String, Object> infoImage = convertRestImpl.getInformationImage(token);
	  
	  List<String> targets = new ArrayList<>();	  
	  
	  targets.addAll(extractIdsFromOutputs(info));	  
	  for (String target: targets) {
		  String result = convertRestImpl.convert(token, converter, target, content).toString();
		  assertTrue(result.length() > 0);
	  }
	  
	  targets.clear();
	  targets.addAll(extractIdsFromOutputs(infoImage));	  
	  for (String target: targets) {
		  String result = convertRestImpl.converToImage(token, converter, target, content).toString();
		  assertTrue(result.length() > 0);
	  }
  }
  
  @Test
  public void testCelDesigner2All() throws Exception {
	  
	  try {
	    	File file = new File("testFiles/convert/sample-cd.xml");
	    	String content = FileUtils.readFileToString(file);
	    	test2All(content, "CellDesigner_SBML");
	    	
	    } catch (Exception e) {
	      e.printStackTrace();
	      throw e;
	    }
  }
  
  @Test
  public void testSbml2All() throws Exception {
	  
	  try {
	    	File file = new File("testFiles/convert/sample-sbml.xml");
	    	String content = FileUtils.readFileToString(file);
	    	test2All(content, "SBML");
	    	
	    } catch (Exception e) {
	      e.printStackTrace();
	      throw e;
	    }
  }
  
  @Test
  public void testSbgn2All() throws Exception {
	  
	  try {
	    	File file = new File("testFiles/convert/sample.sbgn");
	    	String content = FileUtils.readFileToString(file);
	    	test2All(content, "SBGN-ML");
	    	
	    } catch (Exception e) {
	      e.printStackTrace();
	      throw e;
	    }
  }
  
}
