package lcsb.mapviewer.api.projects.overlays;

import static org.junit.Assert.assertNotNull;

import java.util.Map;

import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import com.google.gson.Gson;

import lcsb.mapviewer.api.RestTestFunctions;
import lcsb.mapviewer.model.Project;
import lcsb.mapviewer.model.map.layout.ColorSchemaType;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.user.ObjectPrivilege;
import lcsb.mapviewer.model.user.Privilege;
import lcsb.mapviewer.model.user.PrivilegeType;
import lcsb.mapviewer.model.user.User;
import lcsb.mapviewer.persist.dao.ProjectDao;
import lcsb.mapviewer.services.interfaces.IProjectService;

public class OverlayRestImplTest extends RestTestFunctions {

  @Autowired
  OverlayRestImpl overlayRest;

  @Autowired
  ProjectDao projectDao;

  @Autowired
  IProjectService projectService;

  String projectId = "test_id";

  @Test
  public void testAddDataOverlay() throws Exception {
    try {
      User admin = userService.getUserByLogin("admin");
      Project project = createProject("testFiles/model/sample.xml", projectId);
      userService.setUserPrivilege(admin, new Privilege(100, PrivilegeType.CUSTOM_LAYOUTS, admin));
      userService.setUserPrivilege(admin, new ObjectPrivilege(project, 1, PrivilegeType.VIEW_PROJECT, admin));

      Map<String, Object> result = overlayRest.addOverlay(adminToken, projectId, "x", "desc", "s1", null, null,
          ColorSchemaType.GENERIC.name(), "true");
      Gson gson = new Gson();
      assertNotNull(gson.toJson(result));
    } catch (Exception e) {
      e.printStackTrace();
      throw e;
    } finally {
      projectService.removeProject(projectDao.getProjectByProjectId(projectId), null, false, adminToken);
    }
  }

  private Project createProject(String string, String projectId) throws Exception {
    Project project = new Project(projectId);
    Model model = super.getModelForFile(string, false);
    project.addModel(model);
    projectDao.add(project);
    return project;
  }

}
